/************************************************************************/
/*                                                                      */
/*                      Interrupt driven buffered UART                  */
/*                                                                      */
/*              Author: Peter Dannegger                                 */
/*                                                                      */
/************************************************************************/
// 20110814 -us
// 20110926 -us
// 20111110 -us

#ifndef _uart0_h_
#define _uart0_h_

#include <avr/pgmspace.h>
#include <avr/io.h>
#include <inttypes.h>

// #include "main.h"

#ifndef SBIT
// 			Access bits like variables:
struct bits {
  uint8_t b0:1;
  uint8_t b1:1;
  uint8_t b2:1;
  uint8_t b3:1;
  uint8_t b4:1;
  uint8_t b5:1;
  uint8_t b6:1;
  uint8_t b7:1;
} __attribute__((__packed__));

#define SBIT_(port,pin) ((*(volatile struct bits*)&port).b##pin)
#define	SBIT(x,y)	SBIT_(x,y)

#endif

#ifndef vu8
# define vu8(x)  (*(volatile uint8_t*)&(x))
#endif

#ifndef vu16
# define vu16(x)  (*(volatile uint16_t*)&(x))
#endif

// #define UART_BAUD_SELECT(baudRate,xtalCpu) (uint16_t)((xtalCpu) *1.0 / ((baudRate) * 16UL) - 0.5)
#define UART_BAUD_SELECT(baudRate)	(uint16_t)(((uint32_t)(F_CPU) - 8UL * (baudRate) ) / ( 16UL * (baudRate) )  )

// wenn wir Zeichen über die serielle Schnittstelle empfangen wollen,
// dann muss "USE_UART0_RX" definiert werden.
#define USE_UART0_RX

// wenn wir Zeichen über die serielle Schnittstelle senden wollen,
// dann muss "USE_UART0_TX" definiert werden.
#define USE_UART0_TX
					// size must be in range 2 .. 256
#ifdef USE_UART0_RX
#ifdef UART_RX_BUFFER_SIZE
# define RX0_SIZE	UART_RX_BUFFER_SIZE
#else
# define RX0_SIZE	16		// usable: RX0_SIZE + 2 (4 .. 258)
#endif
#endif

#ifdef USE_UART0_TX
#ifdef UART_TX_BUFFER_SIZE
# define TX0_SIZE	UART_TX_BUFFER_SIZE
#else
# define TX0_SIZE	16		// usable: TX0_SIZE + 1 (3 .. 257)
#endif
#endif

#define	uputs0(x)	uputs0_((uint8_t*)(x))	// avoid char warning
#define uputs0_P(__s)	uputs0_p(PSTR(__s))

#ifdef USE_UART0_RX
extern uint8_t ukbhit0( void );				// 0 = rx buffer empty
extern uint8_t ugetchar0( void );			// get received byte
#endif

#ifdef USE_UART0_TX
extern uint8_t utx0_ready( void );			// 0 = tx still busy
extern void uputchar0( uint8_t c );			// send byte
extern void uputs0_( uint8_t *s );			// send string from SRAM
extern void uputs0_p( const char * pgm_s );
#endif

extern void init_uart0( uint16_t bauddivider );


#define	UTX0_IEN	SBIT( UCSR0B, UDRIE0 )
#define	URX0_IEN	SBIT( UCSR0B, RXCIE0 )


/******************************	Resolve AVR naming chaos ****************/

//-----------------------------	Interrupt vectors -----------------------/
#ifndef	USART0_RX_vect
#if defined 	USART_RX_vect
#define		USART0_RX_vect	USART_RX_vect
#elif defined	USART_RXC_vect
#define		USART0_RX_vect	USART_RXC_vect
#elif defined	USART0_RXC_vect
#define		USART0_RX_vect	USART0_RXC_vect
#endif
#endif

#if !defined USART0_UDRE_vect && defined USART_UDRE_vect
#define	USART0_UDRE_vect USART_UDRE_vect
#endif

//-----------------------------	Register names --------------------------/
#ifndef	UCSR0A
#define	UCSR0A	UCSRA
#endif
#ifndef	UCSR0B
#define	UCSR0B	UCSRB
#endif
#ifndef	UCSR0C
#define	UCSR0C	UCSRC
#endif
#ifndef	UDR0
#define	UDR0	UDR
#endif
#ifndef	UBRR0L
#define	UBRR0L	UBRRL
#endif
#ifndef	UBRR0H
#define	UBRR0H	UBRRH
#endif

//-----------------------------	Bit names -------------------------------/
#ifndef	UCSZ00
#define	UCSZ00	UCSZ0
#endif
#ifndef	UCSZ01
#define	UCSZ01	UCSZ1
#endif
#if !defined URSEL0 && defined URSEL
#define	URSEL0	URSEL
#endif
#ifndef	RXEN0
#define	RXEN0	RXEN
#endif
#ifndef	TXEN0
#define	TXEN0	TXEN
#endif
#ifndef	UDRIE0
#define	UDRIE0	UDRIE
#endif
#ifndef	RXCIE0
#define	RXCIE0	RXCIE
#endif
#ifndef UPM00
#define UPM00	UPM0
#endif
#ifndef UPM01
#define UPM01	UPM1
#endif

#endif
